package co.ab180.airbridge.flutter;

import android.content.Context;
import android.util.Log;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;

import co.ab180.airbridge.AirbridgeConfig;
import co.ab180.dependencies.com.google.gson.JsonElement;
import co.ab180.dependencies.com.google.gson.JsonObject;
import co.ab180.dependencies.com.google.gson.JsonParser;
import co.ab180.dependencies.com.google.gson.JsonPrimitive;
import io.flutter.FlutterInjector;

public class ConfigReader {

    private static final String FILE_NAME = "assets/airbridge.json";

    public AirbridgeConfig build(Context context, String appName, String appToken) {
        AirbridgeConfig.Builder builder = new AirbridgeConfig.Builder(appName, appToken);
        String configJsonString = readConfigFromAssets(context);
        try {
            JsonObject configJson = JsonParser.parseString(configJsonString).getAsJsonObject();

            JsonElement sessionTimeoutSecondsProperty = configJson.get("sessionTimeoutSeconds");
            if (sessionTimeoutSecondsProperty != null && sessionTimeoutSecondsProperty.isJsonPrimitive()) {
                JsonPrimitive primitive = sessionTimeoutSecondsProperty.getAsJsonPrimitive();
                if (primitive.isNumber()) {
                    builder.setSessionTimeoutSeconds(primitive.getAsLong());
                }
            }

            JsonElement autoStartTrackingEnabledProperty = configJson.get("autoStartTrackingEnabled");
            if (autoStartTrackingEnabledProperty != null && autoStartTrackingEnabledProperty.isJsonPrimitive()) {
                JsonPrimitive primitive = autoStartTrackingEnabledProperty.getAsJsonPrimitive();
                if (primitive.isBoolean()) {
                    builder.setAutoStartTrackingEnabled(primitive.getAsBoolean());
                }
            }

            JsonElement userInfoHashEnabledProperty = configJson.get("userInfoHashEnabled");
            if (userInfoHashEnabledProperty != null && userInfoHashEnabledProperty.isJsonPrimitive()) {
                JsonPrimitive primitive = userInfoHashEnabledProperty.getAsJsonPrimitive();
                if (primitive.isBoolean()) {
                    builder.setUserInfoHashEnabled(primitive.getAsBoolean());
                }
            }

            JsonElement trackAirbridgeLinkOnlyProperty = configJson.get("trackAirbridgeLinkOnly");
            if (trackAirbridgeLinkOnlyProperty != null && trackAirbridgeLinkOnlyProperty.isJsonPrimitive()) {
                JsonPrimitive primitive = trackAirbridgeLinkOnlyProperty.getAsJsonPrimitive();
                if (primitive.isBoolean()) {
                    builder.setTrackAirbridgeLinkOnly(primitive.getAsBoolean());
                }
            }

            JsonElement locationCollectionEnabledProperty = configJson.get("locationCollectionEnabled");
            if (locationCollectionEnabledProperty != null && locationCollectionEnabledProperty.isJsonPrimitive()) {
                JsonPrimitive primitive = locationCollectionEnabledProperty.getAsJsonPrimitive();
                if (primitive.isBoolean()) {
                    builder.setLocationCollectionEnabled(primitive.getAsBoolean());
                }
            }

            JsonElement facebookDeferredAppLinkEnabledProperty = configJson.get("facebookDeferredAppLinkEnabled");
            if (facebookDeferredAppLinkEnabledProperty != null && facebookDeferredAppLinkEnabledProperty.isJsonPrimitive()) {
                JsonPrimitive primitive = facebookDeferredAppLinkEnabledProperty.getAsJsonPrimitive();
                if (primitive.isBoolean()) {
                    builder.setFacebookDeferredAppLinkEnabled(primitive.getAsBoolean());
                }
            }
        } catch (Exception ignored) { }

        builder.setPlatform("flutter");

        return builder.build();
    }

    private String readConfigFromAssets(Context context) {
        String configFileFullPath = FlutterInjector.instance().flutterLoader().getLookupKeyForAsset(FILE_NAME);
        InputStream is = null;
        BufferedReader reader = null;
        StringBuilder sb = new StringBuilder();
        try {
            is = context.getAssets().open(configFileFullPath);
            reader = new BufferedReader(new InputStreamReader(is, "UTF-8"));
            String line;
            while ((line = reader.readLine()) != null) {
                sb.append(line);
            }
        } catch (IOException e) {
            Log.d("AirbridgeFlutter", "Could not find \"" + FILE_NAME + "\" file");
        } finally {
            if (is != null) try {
                is.close();
            } catch (IOException ignored) { }

            if (reader != null) try {
                reader.close();
            } catch (IOException ignored) { }
        }
        return sb.toString();
    }
}
