package co.ab180.airbridge.flutter;

import android.content.Intent;
import android.net.Uri;
import android.os.Handler;
import android.os.Looper;

import co.ab180.airbridge.Airbridge;
import co.ab180.airbridge.AirbridgeCallback;
import io.flutter.plugin.common.EventChannel;
import io.flutter.plugin.common.PluginRegistry;

public class DeeplinkAPI implements EventChannel.StreamHandler {

    private static String startDeeplink;
    private static DeeplinkAPI instance;

    private EventChannel.EventSink deeplinkEventSink;
    private Handler mainHandler = new Handler(Looper.getMainLooper());

    static void processDeeplinkData(Intent intent) {
        Airbridge.getDeeplink(intent, new AirbridgeCallback.SimpleCallback<Uri>() {
            @Override
            public void onSuccess(Uri uri) {
                if (instance != null && instance.deeplinkEventSink != null) {
                    instance.deeplinkEventSink.success(uri.toString());
                } else {
                    startDeeplink = uri.toString();
                }
            }

            @Override
            public void onFailure(Throwable throwable) {
                throwable.printStackTrace();
            }
        });
    }

    public static void registerWith(PluginRegistry.Registrar registrar) {
        instance = new DeeplinkAPI();

        final EventChannel deeplinkEvent = new EventChannel(
                registrar.messenger(),
                "airbridge_flutter_sdk/event/deeplink"
        );
        deeplinkEvent.setStreamHandler(instance);
    }

    @Override
    public void onListen(Object o, final EventChannel.EventSink eventSink) {
        String target = Get.type(String.class, o);
        if (target == null || eventSink == null) {
            return;
        }

        if (target.equals("deeplink")) {
            deeplinkEventSink = eventSink;
            if (startDeeplink != null) {
                mainHandler.post(new Runnable() {
                    @Override
                    public void run() {
                        deeplinkEventSink.success(startDeeplink);
                        startDeeplink = null;
                    }
                });
            }
        }
    }

    @Override
    public void onCancel(Object o) {
        String target = Get.type(String.class, o);
        if (target == null) {
            return;
        }

        if (target.equals("deeplink")) {
            deeplinkEventSink = null;
        }
    }
}