package co.ab180.airbridge.flutter;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.annotation.Nonnull;
import javax.annotation.Nullable;

import co.ab180.airbridge.Airbridge;
import co.ab180.airbridge.event.Event;
import co.ab180.airbridge.event.ecommerce.AddToCartEvent;
import co.ab180.airbridge.event.ecommerce.HomeViewEvent;
import co.ab180.airbridge.event.ecommerce.OrderCompletedEvent;
import co.ab180.airbridge.event.ecommerce.ProductDetailsViewEvent;
import co.ab180.airbridge.event.ecommerce.ProductListViewEvent;
import co.ab180.airbridge.event.ecommerce.SearchResultViewEvent;
import co.ab180.airbridge.event.model.Product;
import co.ab180.airbridge.event.user.SignInEvent;
import co.ab180.airbridge.event.user.SignOutEvent;
import co.ab180.airbridge.event.user.SignUpEvent;
import io.flutter.plugin.common.MethodCall;
import io.flutter.plugin.common.MethodChannel;
import io.flutter.plugin.common.PluginRegistry;

public class EventAPI implements MethodChannel.MethodCallHandler {

    //
    // flutter plugin setting
    //

    public static void registerWith(PluginRegistry.Registrar registrar) {
        final MethodChannel channel = new MethodChannel(
                registrar.messenger(),
                "airbridge_flutter_sdk/method/event"
        );
        channel.setMethodCallHandler(new EventAPI());
    }

    @Override
    public void onMethodCall(MethodCall call, MethodChannel.Result result) {
        Map<?, ?> arguments = Get.type(HashMap.class, call.arguments);

        switch (call.method) {
            case "sendSignIn":
                sendSignIn(arguments, result);
                break;
            case "sendSignUp":
                sendSignUp(arguments, result);
                break;
            case "sendSignOut":
                sendSignOut(arguments, result);
                break;
            case "sendViewHome":
                sendViewHome(arguments, result);
                break;
            case "sendViewSearchResult":
                sendViewSearchResult(arguments, result);
                break;
            case "sendViewProductList":
                sendViewProductList(arguments, result);
                break;
            case "sendViewProductDetail":
                sendViewProductDetail(arguments, result);
                break;
            case "sendAddToCart":
                sendAddToCart(arguments, result);
                break;
            case "sendPurchase":
                sendPurchase(arguments, result);
                break;
            case "sendCustom":
                sendCustom(arguments, result);
                break;
            default:
                result.notImplemented();
                break;
        }
    }

    //
    // flutter method
    //

    private void sendSignIn(Map<?, ?> arguments, MethodChannel.Result result) {
        Map<?, ?> user = Get.type(HashMap.class, arguments, "user");
        Map<?, ?> option = Get.type(HashMap.class, arguments, "option");

        String id = Get.type(String.class, user, "id");
        String email = Get.type(String.class, user, "email");
        String phone = Get.type(String.class, user, "phone");
        Map<String, String> alias = Get.type(HashMap.class, user, "alias");
        Map<String, Object> attributes = Get.type(HashMap.class, user, "attributes");
        // TODO : After support double type of attribute value, it will be legacy code
        // **** Caution ****
        // Flutter gives only double type decimal values.
        // Airbridge attribute accepts only float type of decimal number.
        // It should change double type to float type for proper work.
        for (String key : attributes.keySet()) {
            Object value = attributes.get(key);
            if (value instanceof Double) {
                attributes.put(key, ((Double) value).floatValue());
            }
        }

        SignInEvent event = new SignInEvent();

        addOptionToEvent(event, option);
        event.setId(id);
        event.setEmail(email);
        event.setPhone(phone);
        event.setUserAlias(alias);
        event.setUserAttributes(attributes);

        Airbridge.trackEvent(event);

        result.success(null);
    }

    private void sendSignUp(Map<?, ?> arguments, MethodChannel.Result result) {
        Map<?, ?> user = Get.type(HashMap.class, arguments, "user");
        Map<?, ?> option = Get.type(HashMap.class, arguments, "option");

        String id = Get.type(String.class, user, "id");
        String email = Get.type(String.class, user, "email");
        String phone = Get.type(String.class, user, "phone");
        Map<String, String> alias = Get.type(HashMap.class, user, "alias");
        Map<String, Object> attributes = Get.type(HashMap.class, user, "attributes");
        // TODO : After support double type of attribute value, it will be legacy code
        // **** Caution ****
        // Flutter gives only double type decimal values.
        // Airbridge attribute accepts only float type of decimal number.
        // It should change double type to float type for proper work.
        for (String key : attributes.keySet()) {
            Object value = attributes.get(key);
            if (value instanceof Double) {
                attributes.put(key, ((Double) value).floatValue());
            }
        }

        SignUpEvent event = new SignUpEvent();

        addOptionToEvent(event, option);
        event.setId(id);
        event.setEmail(email);
        event.setPhone(phone);
        event.setUserAlias(alias);
        event.setUserAttributes(attributes);

        Airbridge.trackEvent(event);

        result.success(null);
    }

    private void sendSignOut(Map<?, ?> arguments, MethodChannel.Result result) {
        Map<?, ?> option = Get.type(HashMap.class, arguments, "option");

        SignOutEvent event = new SignOutEvent();

        addOptionToEvent(event, option);

        Airbridge.trackEvent(event);

        result.success(null);
    }

    private void sendViewHome(Map<?, ?> arguments, MethodChannel.Result result) {
        Map<?, ?> option = Get.type(HashMap.class, arguments, "option");
        Event event = new HomeViewEvent();
        addOptionToEvent(event, option);
        Airbridge.trackEvent(event);
        result.success(null);
    }

    private void sendViewSearchResult(Map<?, ?> arguments, MethodChannel.Result result) {
        Map<?, ?> option = Get.type(HashMap.class, arguments, "option");
        Event event = new SearchResultViewEvent();
        addOptionToEvent(event, option);
        Airbridge.trackEvent(event);
        result.success(null);
    }

    private void sendViewProductList(Map<?, ?> arguments, MethodChannel.Result result) {
        Map<?, ?> option = Get.type(HashMap.class, arguments, "option");
        Event event = new ProductListViewEvent();
        addOptionToEvent(event, option);
        Airbridge.trackEvent(event);
        result.success(null);
    }

    private void sendViewProductDetail(Map<?, ?> arguments, MethodChannel.Result result) {
        Map<?, ?> option = Get.type(HashMap.class, arguments, "option");
        Event event = new ProductDetailsViewEvent();
        addOptionToEvent(event, option);
        Airbridge.trackEvent(event);
        result.success(null);
    }

    private void sendAddToCart(Map<?, ?> arguments, MethodChannel.Result result) {
        Map<?, ?> option = Get.type(HashMap.class, arguments, "option");
        Event event = new AddToCartEvent();
        addOptionToEvent(event, option);
        Airbridge.trackEvent(event);
        result.success(null);
    }

    private void sendPurchase(Map<?, ?> arguments, MethodChannel.Result result) {
        Map<?, ?> option = Get.type(HashMap.class, arguments, "option");
        Event event = new OrderCompletedEvent();
        addOptionToEvent(event, option);
        Airbridge.trackEvent(event);
        result.success(null);
    }

    private void sendCustom(Map<?, ?> arguments, MethodChannel.Result result) {
        String category = Get.type(String.class, arguments, "category");
        Map<?, ?> option = Get.type(HashMap.class, arguments, "option");
        Event event = new Event(category);
        addOptionToEvent(event, option);
        Airbridge.trackEvent(event);
        result.success(null);
    }

    //
    // private - tool
    //

    /**
     * Add option infomation to event
     * <p>
     * add null when some data has unmatch type
     * @param       event
     *                  target event
     * @param       option
     *                  json-object which has option infomation
     */
    private void addOptionToEvent(@Nonnull Event event, @Nonnull Map option) {
        String action = Get.type(String.class, option, "action");
        String label = Get.type(String.class, option, "label");
        Float value = Get.type(Float.class, Get.type(Number.class, option, "value"));
        Map<String, Object> attributes = Get.type(HashMap.class, option, "attributes");
        Map<String, Object> semantics = Get.type(HashMap.class, option, "semantics");

        event.setAction(action);
        event.setLabel(label);
        event.setValue(value);

        if (attributes != null) {
            event.setCustomAttributes(attributes);
        }

        if (semantics != null) {
            event.setSemanticAttributes(semantics);
        }
    }

    /**
     * convert array-of-maps to array-of-Products
     * <p>
     * return null when inputs is null
     * <p>
     * add null when some data has unmatch type
     * @param       inputs
     *                  json-array of json-objects
     * @return		array of Products
     */
    @Nullable
    private List<Product> convertProducts(@Nullable List<?> inputs) {
        if (inputs == null) return null;

        List<Product> outputs = new ArrayList<>();

        for (int i=0; i<inputs.size(); i++) {
            HashMap<?, ?> object = Get.type(HashMap.class, inputs, i);

            Product product = convertProduct(object);

            outputs.add(product);
        }

        return outputs;
    }

    /**
     * convert map to Product
     * <p>
     * return null when input is null
     * <p>
     * add null when some data has unmatch type
     * @param       input
     *                  json-object which has product infomation
     * @return		Product
     */
    @Nullable
    private Product convertProduct(@Nullable Map<?, ?> input) {
        if (input == null) return null;

        Product output = new Product();

        String id = Get.type(String.class, input, "id");
        String name = Get.type(String.class, input, "name");
        String currency = Get.type(String.class, input, "currency");
        Integer quantity = Get.type(Integer.class, Get.type(Number.class, input, "quantity"));
        Float price = Get.type(Float.class, Get.type(Number.class, input, "price"));
        Integer position = Get.type(Integer.class, Get.type(Number.class, input, "position"));

        output.setId(id);
        output.setName(name);
        output.setCurrency(currency);
        output.setQuantity(quantity);
        output.setPrice(price);
        output.setPosition(position);

        return output;
    }
}
