package co.ab180.airbridge.flutter;

import java.util.HashMap;
import java.util.Map;

import co.ab180.airbridge.AdvertisingIdInfo;
import co.ab180.airbridge.Airbridge;
import co.ab180.airbridge.AirbridgeCallback;
import io.flutter.plugin.common.MethodCall;
import io.flutter.plugin.common.MethodChannel;
import io.flutter.plugin.common.PluginRegistry;

public class StateAPI implements MethodChannel.MethodCallHandler {

    //
    // flutter plugin setting
    //

    public static void registerWith(PluginRegistry.Registrar registrar) {
        final MethodChannel channel = new MethodChannel(
                registrar.messenger(),
                "airbridge_flutter_sdk/method/state"
        );
        channel.setMethodCallHandler(new StateAPI());
    }

    @Override
    public void onMethodCall(MethodCall call, MethodChannel.Result result) {
        Map<?, ?> arguments = Get.type(HashMap.class, call.arguments);

        if (call.method.equals("setUser")) {
            setUser(arguments, result);
        } else if (call.method.equals("updateUser")) {
            updateUser(arguments, result);
        } else if (call.method.equals("startTracking")) {
            startTracking(result);
        } else if (call.method.equals("getDeviceUUID")) {
            getDeviceUUID(result);
        } else {
            result.notImplemented();
        }
    }

    //
    // flutter method
    //

    private void setUser(Map<?, ?> arguments, MethodChannel.Result result) {

        String id = Get.type(String.class, arguments, "id");
        String email = Get.type(String.class, arguments, "email");
        String phone = Get.type(String.class, arguments, "phone");
        Map<?, ?> alias = Get.type(HashMap.class, arguments, "alias");
        Map<?, ?> attributes = Get.type(HashMap.class, arguments, "attributes");

        Airbridge.getCurrentUser().setId(id);
        Airbridge.getCurrentUser().setEmail(email);
        Airbridge.getCurrentUser().setPhone(phone);

        if (alias != null) {
            for (Object key : alias.keySet()) {
                if (key instanceof String) {
                    String value = Get.type(String.class, alias, (String) key);
                    Airbridge.getCurrentUser().setAlias((String) key, value);
                }
            }
        } else {
            Airbridge.getCurrentUser().clearAlias();
        }

        if (attributes != null) {
            for (Object key : attributes.keySet()) {
                if (key instanceof String) {
                    Object value = attributes.get(key);
                    if (value instanceof Integer) {
                        Airbridge.getCurrentUser().setAttribute((String) key, (int) value);
                    } else if (value instanceof Long) {
                        Airbridge.getCurrentUser().setAttribute((String) key, (long) value);
                    } else if (value instanceof Float || value instanceof Double) {
                        Airbridge.getCurrentUser().setAttribute((String) key, ((Number) value).floatValue());
                    } else if (value instanceof Boolean) {
                        Airbridge.getCurrentUser().setAttribute((String) key, (boolean) value);
                    } else if (value instanceof String) {
                        Airbridge.getCurrentUser().setAlias((String) key, (String) value);
                    }
                }
            }
        } else {
            Airbridge.getCurrentUser().clearAttributes();
        }


        result.success(null);
    }

    private void updateUser(Map<?, ?> arguments, MethodChannel.Result result) {
        String id = Get.type(String.class, arguments, "id");
        String email = Get.type(String.class, arguments, "email");
        String phone = Get.type(String.class, arguments, "phone");
        Map<?, ?> alias = Get.type(HashMap.class, arguments, "alias");
        Map<?, ?> attributes = Get.type(HashMap.class, arguments, "attributes");

        if (id != null) {
            Airbridge.getCurrentUser().setId(id);
        }
        if (email != null) {
            Airbridge.getCurrentUser().setEmail(email);
        }
        if (phone != null) {
            Airbridge.getCurrentUser().setPhone(phone);
        }
        if (alias != null) {
            for (Object key : alias.keySet()) {
                if (key instanceof String) {
                    String value = Get.type(String.class, alias, (String) key);
                    Airbridge.getCurrentUser().setAlias((String) key, value);
                }
            }
        }
        if (attributes != null) {
            for (Object key : attributes.keySet()) {
                if (key instanceof String) {
                    Object value = attributes.get(key);
                    if (value instanceof Integer) {
                        Airbridge.getCurrentUser().setAttribute((String) key, (int) value);
                    } else if (value instanceof Long) {
                        Airbridge.getCurrentUser().setAttribute((String) key, (long) value);
                    } else if (value instanceof Float || value instanceof Double) {
                        Airbridge.getCurrentUser().setAttribute((String) key, ((Number) value).floatValue());
                    } else if (value instanceof Boolean) {
                        Airbridge.getCurrentUser().setAttribute((String) key, (boolean) value);
                    } else if (value instanceof String) {
                        Airbridge.getCurrentUser().setAlias((String) key, (String) value);
                    }
                }
            }
        }

        result.success(null);
    }

    private void startTracking(MethodChannel.Result result) {
        Airbridge.startTracking();
        result.success(null);
    }

    private void getDeviceUUID(final MethodChannel.Result result) {
        Airbridge.getDeviceInfo().getUUID(new AirbridgeCallback.SimpleCallback<String>() {
            @Override
            public void onSuccess(String uuid) {
                result.success(uuid);
            }

            @Override
            public void onFailure(Throwable throwable) {
                result.success(null);
            }
        });
    }

    private void getIsLimitADTracking(final MethodChannel.Result result) {
        Airbridge.getDeviceInfo().getAdvertisingIdInfo(new AirbridgeCallback.SimpleCallback<AdvertisingIdInfo>() {
            @Override
            public void onSuccess(AdvertisingIdInfo info) {
                result.success(info.getLimitAdTrackingEnabled());
            }

            @Override
            public void onFailure(Throwable throwable) {
                result.success(false);
            }
        });
    }
}
