import 'package:airbridge_flutter_sdk_example/storage.dart';
import 'package:flutter/material.dart';

import 'home/home.dart';
import 'log/log.dart';

class App extends StatefulWidget {
    @override
    _AppState createState() => _AppState();
}


class _AppState extends State<App> {
    final navigatorKey = GlobalKey<NavigatorState>();

    Home home = new Home();
    Log log = new Log();

    int _selectedIndex = 0;

    Function deeplinkCallback;

    _AppState() {
        deeplinkCallback = (deeplink) {
            if (deeplink == null) {
                return;
            }

            show('Deeplink', deeplink);
        };
    }

    @override
    void initState() {
        super.initState();

        show('Deeplink', Storage.get('deeplink'));

        Storage.addOnSet('deeplink', deeplinkCallback);
    }

    @override
    void dispose() {
        super.dispose();
        Storage.removeOnSet('deeplink', deeplinkCallback);
    }

    void show(String title, String content) {
        if (content == null) {
            return;
        }

        Future.delayed(
            Duration.zero,
                () {
                showDialog(
                    context: navigatorKey.currentState.overlay.context,
                    builder: (context) {
                        return AlertDialog(
                            title: Text(title),
                            content: Text(content),
                        );
                    },
                );
            }
        );
    }

    @override
    Widget build(BuildContext context) {
        return MaterialApp(
            navigatorKey: navigatorKey,
            home: Scaffold(
                appBar: AppBar(
                    title: const Text('Airbridge Flutter SDK Example'),
                ),
                body: getBody(),
                bottomNavigationBar: BottomNavigationBar(
                    onTap: onTap,
                    items: [
                        BottomNavigationBarItem(
                            icon: Icon(Icons.home),
                            title: Text('Home'),
                        ),
                        BottomNavigationBarItem(
                            icon: Icon(Icons.info),
                            title: Text('Log'),
                        ),
                    ],
                    currentIndex: _selectedIndex,
                ),
            ),
        );
    }

    Widget getBody() {
        switch(_selectedIndex) {
            case 0:
                return home;
                break;
            case 1:
                return log;
                break;
            default:
                return home;
                break;
        }
    }

    void onTap(int index) {
        setState(() {
            _selectedIndex = index;
        });
    }
}