import 'package:airbridge_flutter_sdk_example/basic/status_row.dart' as Basic;
import 'package:airbridge_flutter_sdk_example/storage.dart';
import 'package:flutter/material.dart';

import 'package:flutter/services.dart';

class DeeplinkStatus extends StatefulWidget {
    @override
    State<StatefulWidget> createState() {
        return _State();
    }
}

class _State extends State<DeeplinkStatus> {
    String deeplink = '';
    Function deeplinkCallback;

    _State() {
        deeplinkCallback = (deeplink) => updateDeeplink(deeplink);
    }

    @override
    void initState() {
        super.initState();

        updateDeeplink(Storage.get('deeplink'));

        Storage.addOnSet('deeplink', deeplinkCallback);
    }

    @override
    void dispose() {
        super.dispose();
        Storage.removeOnSet('deeplink', deeplinkCallback);
    }

    void updateDeeplink(String url) {
        if (url != null) {
            deeplink = url;
            setState(() {});
        }
    }

    @override
    Widget build(BuildContext context) {
        return Container(
            padding: EdgeInsets.only(left: 6.0, right: 6.0),
            child: Column(
                children: [
                    _StatusRow(
                        name: 'Deeplink',
                        status: deeplink,
                        onPressed: () {
                            Clipboard.setData(
                                ClipboardData(text: deeplink)
                            );
                            Scaffold.of(context).showSnackBar(SnackBar(
                                content: Text('Deeplink is copied!'),
                                duration: Duration(seconds: 1),
                            ));
                        },
                    ),
                ],
            ),
        );
    }
}

class _StatusRow extends StatelessWidget {
    final String name;
    final String status;
    final Function onPressed;

    _StatusRow({
        this.name,
        this.status,
        this.onPressed,
    });

    @override
    Widget build(BuildContext context) {
        return Basic.StatusRow(
            name: name,
            status: status,
            backColor: Colors.lightBlue,
            textColor: Colors.white,
            onPressed: onPressed,
        );
    }
}