import 'package:airbridge_flutter_sdk_example/basic/status_row.dart' as Basic;
import 'package:airbridge_flutter_sdk_example/storage.dart';
import 'package:flutter/material.dart';

import 'package:flutter/services.dart';

class DeviceStatus extends StatefulWidget {
    @override
    State<StatefulWidget> createState() {
        return _State();
    }
}

class _State extends State<DeviceStatus> {
    Function deviceUUIDCallback;
    Function latCallback;

    String deviceUUID = '';
    bool isLimitADTracking;

    _State() {
        deviceUUIDCallback = (deviceUUID) {
            this.deviceUUID = deviceUUID;
            setState(() {});
        };

        latCallback = (isLimitADTracking) {
            this.isLimitADTracking = isLimitADTracking;
            setState(() {});
        };
    }

    @override
    void initState() {
        super.initState();

        deviceUUID = Storage.get('deviceUUID') ?? '';
        isLimitADTracking = Storage.get('isLimitADTracking') ?? false;

        Storage.addOnSet('deviceUUID', deviceUUIDCallback);
        Storage.addOnSet('isLimitADTracking', latCallback);

        setState(() {});
    }

    @override
    void dispose() {
        super.dispose();

        Storage.removeOnSet('deviceUUID', deviceUUIDCallback);
        Storage.removeOnSet('isLimitADTracking', latCallback);
    }

    @override
    Widget build(BuildContext context) {
        return Container(
            child: Column(
                children: [
                    _StatusRow(
                        name: 'Device UUID',
                        status: deviceUUID,
                        onPressed: () {
                            Clipboard.setData(
                                ClipboardData(text: deviceUUID)
                            );
                            Scaffold.of(context).showSnackBar(SnackBar(
                                content: Text('Device UUID is copied!'),
                                duration: Duration(seconds: 1),
                            ));
                        },
                    ),
                    _StatusRow(
                        name: 'LAT',
                        status: '$isLimitADTracking',
                        onPressed: () {},
                    ),
                ],
            )
        );
    }
}

class _StatusRow extends StatelessWidget {
    final String name;
    final String status;
    final Function onPressed;

    _StatusRow({
        this.name,
        this.status,
        this.onPressed
    });

    @override
    Widget build(BuildContext context) {
        return Basic.StatusRow(
            name: name,
            status: status,
            backColor: Colors.teal,
            textColor: Colors.white,
            onPressed: onPressed,
        );
    }
}