import 'package:airbridge_flutter_sdk/airbridge.dart';
import 'package:airbridge_flutter_sdk_example/basic/button.dart' as Basic;
import 'package:airbridge_flutter_sdk_example/storage.dart';
import 'package:flutter/material.dart';

import 'package:flutter/services.dart';

class PlacementTest extends StatefulWidget {
    @override
    State<StatefulWidget> createState() {
        return _State();
    }
}

class _State extends State<PlacementTest> {
    final _formKey = GlobalKey<FormState>();
    final textController = TextEditingController();

    @override
    void initState() {
        super.initState();

        String trackingLink = Storage.get('trackingLink');
        if (trackingLink != null) {
            textController.text = trackingLink;
        }
        setState(() {});
    }

    @override
    Widget build(BuildContext context) {
        return Container(
            padding: EdgeInsets.only(left: 6.0, right: 6.0),
            child: Column(
                children: [
                    Form(
                        key: _formKey,
                        child: TextFormField(
                            controller: textController,
                            keyboardType: TextInputType.url,
                            decoration: InputDecoration(
                                hintText: 'Enter tracking link'
                            ),
                            validator: (text) {
                                Uri uri = Uri.tryParse(text);

                                if (uri == null
                                    || (uri.scheme != 'https'
                                        && uri.scheme != 'http'))
                                {
                                    return 'not tracking link';
                                }

                                return null;
                            },
                            onChanged: (text) {
                                Storage.set('trackingLink', text);
                            },
                        ),
                    ),
                    Container(
                        padding: EdgeInsets.all(6.0),
                        child: Row(
                            mainAxisAlignment: MainAxisAlignment.end,
                            children: [
                                _Button(
                                    'Impression',
                                    onPressed: () => this.impression(),
                                ),
                                _Button(
                                    'Click',
                                    onPressed: () => this.click(),
                                )
                            ],
                        ),
                    )
                ],
            ),
        );
    }

    void impression() {
        if (_formKey.currentState.validate()) {
            Airbridge.placement.impression(textController.text);
        }
    }

    void click() {
        if (_formKey.currentState.validate()) {
            Airbridge.placement.click(textController.text);
        }
    }
}

class _Button extends StatelessWidget {
    final String name;
    final Function onPressed;

    _Button(
        this.name, {
        this.onPressed,
    });

    @override
    Widget build(BuildContext context) {
        return SizedBox(
            width: 100,
            child: Basic.Button(
                name,
                backColor: Colors.lightGreen,
                textColor: Colors.white,
                onPressed: onPressed,
            ),
        );
    }
}