import 'package:airbridge_flutter_sdk_example/basic/button_grid.dart';
import 'package:flutter/material.dart';

import 'package:airbridge_flutter_sdk/airbridge.dart';

class SendEventGrid extends StatelessWidget {
    @override
    Widget build(BuildContext context) {
        return ButtonGrid(
            width: 3,
            backColor: Colors.indigoAccent,
            textColor: Colors.white,
            datas: [
                ButtonData('viewHome', (){
                    Airbridge.event.send(ViewHomeEvent(
                        option: createOption('viewHome'),
                    ));
                }),
                ButtonData('purchase', (){
                    Airbridge.event.send(PurchaseEvent(
                        option: createOption('purchase'),
                        transactionID: 'transactionID',
                        products: createProducts('purchase'),
                        isInAppPurchase: true,
                        currency: 'KRW',
                        totalValue: 30000,
                        totalQuantity: 10,
                    ));
                }),
                ButtonData('addToCart', (){
                    Airbridge.event.send(AddToCartEvent(
                        option: createOption('addToCart'),
                        cartID: 'cartID',
                        products: createProducts('addToCart'),
                        currency: 'KRW',
                        totalValue: 30000,
                        totalQuantity: 10
                    ));
                }),
                ButtonData('viewProductDetail', (){
                    Airbridge.event.send(ViewProductDetailEvent(
                        option: createOption('viewProductDetail'),
                        products: createProducts('viewProductDetail'),
                    ));
                }),
                ButtonData('viewProductList', (){
                    Airbridge.event.send(ViewProductListEvent(
                        option: createOption('viewProductList'),
                        listID: 'listID',
                        products: createProducts('viewProductList'),
                    ));
                }),
                ButtonData('viewSearchResult', (){
                    Airbridge.event.send(ViewSearchResultEvent(
                        option: createOption('ViewSearchResult'),
                        query: 'query',
                        products: createProducts('viewSearchResult'),
                    ));
                }),
                ButtonData('custom', (){
                    Airbridge.event.send(Event(
                        'custom',
                        option: createOption('custom'),
                    ));
                }),
            ],
        );
    }

    EventOption createOption(String name) {
        if (name == null) {
            name = '';
        }

        return EventOption(
            action: '$name action',
            label: '$name label',
            value: name.length,
            attributes: {
                '$name 0': '0',
                '$name 1': '1',
            }
        );
    }

    List<Product> createProducts(String name) {
        List<Product> products = List();

        for (int i=0; i<name.length; i++) {
            products.add(Product(
                id: 'id $name $i',
                name: 'name $name $i',
                currency: 'KRW',
                price: 1000 * i,
                quantity: 3 * i,
                position: i,
            ));
        }

        return products;
    }
}