//
//  AFLEventAPI.m
//  airbridge_flutter_sdk
//
//  Created by WOF on 08/10/2019.
//

#import "AFLEventAPI.h"

// Airbridge
#import <AirBridge/ABInAppEvent.h>
#import <AirBridge/ABEcommerceEvent.h>
#import <AirBridge/ABUserEvent.h>
// AB
#import "AFLGet.h"

@implementation AFLEventAPI

//
// flutter plugin setting
//

+ (void) registerWithRegistrar:(NSObject<FlutterPluginRegistrar>*)registrar {
    FlutterMethodChannel* channel = [FlutterMethodChannel methodChannelWithName:@"airbridge_flutter_sdk/method/event"
                                                                binaryMessenger:registrar.messenger];
    AFLEventAPI* instance = [[AFLEventAPI alloc] init];
    
    [registrar addMethodCallDelegate:instance channel:channel];
}

- (void) handleMethodCall:(FlutterMethodCall*)call result:(FlutterResult)result {
    NSDictionary* arguments = [AFLGet type:NSDictionary.class object:call.arguments];
    
    if ([@"sendSignIn" isEqualToString:call.method]) {
        [self sendSignIn:arguments result:result];
    } else if ([@"sendSignUp" isEqualToString:call.method]) {
        [self sendSignUp:arguments result:result];
    } else if ([@"sendSignOut" isEqualToString:call.method]) {
        [self sendSignOut:arguments result:result];
    } else if ([@"sendViewHome" isEqualToString:call.method]) {
        [self sendViewHome:arguments result:result];
    } else if ([@"sendViewSearchResult" isEqualToString:call.method]) {
        [self sendViewSearchResult:arguments result:result];
    } else if ([@"sendViewProductList" isEqualToString:call.method]) {
        [self sendViewProductList:arguments result:result];
    } else if ([@"sendViewProductDetail" isEqualToString:call.method]) {
        [self sendViewProductDetail:arguments result:result];
    } else if ([@"sendAddToCart" isEqualToString:call.method]) {
        [self sendAddToCart:arguments result:result];
    } else if ([@"sendPurchase" isEqualToString:call.method]) {
        [self sendPurchase:arguments result:result];
    } else if ([@"sendCustom" isEqualToString:call.method]) {
        [self sendCustom:arguments result:result];
    } else {
        result(FlutterMethodNotImplemented);
    }
}

//
// flutter method
//

- (void) sendSignIn:(nullable NSDictionary*)arguments result:(FlutterResult)result {
    NSDictionary* user = [AFLGet type:NSDictionary.class dictionary:arguments key:@"user"];
    NSDictionary* option = [AFLGet type:NSDictionary.class dictionary:arguments key:@"option"];

    ABUser* nativeUser = convertUser(user);

    ABUserEvent* event = [[ABUserEvent alloc] initWithUser:nativeUser];

    addOptionToEvent(event, option);

    [event sendSignin];
}

- (void) sendSignUp:(nullable NSDictionary*)arguments result:(FlutterResult)result {
    NSDictionary* user = [AFLGet type:NSDictionary.class dictionary:arguments key:@"user"];
    NSDictionary* option = [AFLGet type:NSDictionary.class dictionary:arguments key:@"option"];

    ABUser* nativeUser = convertUser(user);

    ABUserEvent* event = [[ABUserEvent alloc] initWithUser:nativeUser];

    addOptionToEvent(event, option);

    [event sendSignup];
}

- (void) sendSignOut:(nullable NSDictionary*)arguments result:(FlutterResult)result {
    NSDictionary* option = [AFLGet type:NSDictionary.class dictionary:arguments key:@"option"];
    ABUserEvent* event = [[ABUserEvent alloc] init];

    addOptionToEvent(event, option);

    [event expireUser];
}

- (void) sendViewHome:(nullable NSDictionary*)arguments result:(FlutterResult)result {
    NSDictionary* option = [AFLGet type:NSDictionary.class dictionary:arguments key:@"option"];
    ABEcommerceEvent* event = [[ABEcommerceEvent alloc] init];
    addOptionToEvent(event, option);
    [event sendViewHome];
    
    result(nil);
}

- (void) sendViewSearchResult:(nullable NSDictionary*)arguments result:(FlutterResult)result {
    NSDictionary* option = [AFLGet type:NSDictionary.class dictionary:arguments key:@"option"];
    ABEcommerceEvent* event = [[ABEcommerceEvent alloc] init];
    addOptionToEvent(event, option);
    [event sendViewSearchResult];
    
    result(nil);
}

- (void) sendViewProductList:(nullable NSDictionary*)arguments result:(FlutterResult)result {
    NSDictionary* option = [AFLGet type:NSDictionary.class dictionary:arguments key:@"option"];
    ABEcommerceEvent* event = [[ABEcommerceEvent alloc] init];
    addOptionToEvent(event, option);
    [event sendViewProductList];
    
    result(nil);
}

- (void) sendViewProductDetail:(nullable NSDictionary*)arguments result:(FlutterResult)result {
    NSDictionary* option = [AFLGet type:NSDictionary.class dictionary:arguments key:@"option"];
    ABEcommerceEvent* event = [[ABEcommerceEvent alloc] init];
    addOptionToEvent(event, option);
    [event sendViewProductDetail];
    
    result(nil);
}

- (void) sendAddToCart:(nullable NSDictionary*)arguments result:(FlutterResult)result {
    NSDictionary* option = [AFLGet type:NSDictionary.class dictionary:arguments key:@"option"];
    ABEcommerceEvent* event = [[ABEcommerceEvent alloc] init];
    addOptionToEvent(event, option);
    [event sendAddProductToCart];
    
    result(nil);
}

- (void) sendPurchase:(nullable NSDictionary*)arguments result:(FlutterResult)result {
    NSDictionary* option = [AFLGet type:NSDictionary.class dictionary:arguments key:@"option"];
    ABEcommerceEvent* event = [[ABEcommerceEvent alloc] init];
    addOptionToEvent(event, option);
    [event sendCompleteOrder];
    
    result(nil);
}

- (void) sendCustom:(nullable NSDictionary*)arguments result:(FlutterResult)result {
    NSString* category = [AFLGet type:NSString.class dictionary:arguments key:@"category"];
    NSDictionary* option = [AFLGet type:NSDictionary.class dictionary:arguments key:@"option"];
    
    ABInAppEvent* event = [[ABInAppEvent alloc] init];

    addOptionToEvent(event, option);
    [event setCategory:category];
    
    [event send];
}

//
// private - tool
//

/**
 * Add option infomation to event
 * @discussion  add nil when some data has unmatch type
 * @param       event
 *                  target event
 * @param       option
 *                  dictionary which has option infomation
 */
static void addOptionToEvent(ABInAppEvent* event, NSDictionary* __nullable option) {
    if (option == nil) return;
    
    NSString* action = [AFLGet type:NSString.class dictionary:option key:@"action"];
    NSString* label = [AFLGet type:NSString.class dictionary:option key:@"label"];
    NSNumber* value = [AFLGet type:NSNumber.class dictionary:option key:@"value"];
    NSDictionary* attributes = [AFLGet type:NSDictionary.class dictionary:option key:@"attributes"];
    NSDictionary* semantics = [AFLGet type:NSDictionary.class dictionary:option key:@"semantics"];
    
    [event setAction:action];
    [event setLabel:label];
    [event setValue:value];
    [event setCustoms:attributes];
    [event setSemantics:semantics];
}

/**
 * Convert dictionary to ABUser
 * @discussion  return nil when user is nil
 *
 *              add nil when some data has unmatch type
 * @param       user
 *                  dictionary which has user infomation
 * @return      ABUser
 */
static ABUser* __nullable convertUser(NSDictionary* __nullable user) {
    if(user == nil) return nil;
    
    NSString* ID = [AFLGet type:NSString.class dictionary:user key:@"id"];
    NSString* email = [AFLGet type:NSString.class dictionary:user key:@"email"];
    NSString* phone = [AFLGet type:NSString.class dictionary:user key:@"phone"];
    NSDictionary<NSString*, NSString*>* alias = [AFLGet type:NSDictionary.class dictionary:user key:@"alias"];
    NSDictionary<NSString*, NSObject*>* attributes = [AFLGet type:NSDictionary.class dictionary:user key:@"attributes"];
    
    ABUser* nativeUser = [[ABUser alloc] init];
    nativeUser.ID = ID;
    nativeUser.email = email;
    nativeUser.phone = phone;
    nativeUser.alias = alias;
    nativeUser.attributes = attributes;
    
    return nativeUser;
}

@end
