//
//  AFLStateAPI.m
//  airbridge_flutter_sdk
//
//  Created by WOF on 08/10/2019.
//

#import "AFLStateAPI.h"

// Airbridge
#import <AirBridge/AirBridge.h>
// AB
#import "AFLGet.h"

NS_ASSUME_NONNULL_BEGIN

@implementation AFLStateAPI

//
// flutter plugin setting
//

+ (void) registerWithRegistrar:(NSObject<FlutterPluginRegistrar>*)registrar {
    FlutterMethodChannel* channel = [FlutterMethodChannel methodChannelWithName:@"airbridge_flutter_sdk/method/state"
                                                                binaryMessenger:registrar.messenger];
    AFLStateAPI* instance = [[AFLStateAPI alloc] init];
    
    [registrar addMethodCallDelegate:instance channel:channel];
}

- (void) handleMethodCall:(FlutterMethodCall*)call result:(FlutterResult)result {
    NSDictionary* arguments = [AFLGet type:NSDictionary.class object:call.arguments];
    
    if ([@"setUser" isEqualToString:call.method]) {
        [self setUser:arguments result:result];
    } else if ([@"updateUser" isEqualToString:call.method]) {
        [self updateUser:arguments result:result];
    } else if ([@"getDeviceUUID" isEqualToString:call.method]) {
        [self getDeviceUUID:result];
    } else if ([@"startTracking" isEqualToString:call.method]) {
        [self startTracking:result];
    } else {
        result(FlutterMethodNotImplemented);
    }
}

//
// flutter method
//

- (void) setUser:(nullable NSDictionary*)arguments result:(FlutterResult)result {
    NSString* ID = [AFLGet type:NSString.class dictionary:arguments key:@"id"];
    NSString* email = [AFLGet type:NSString.class dictionary:arguments key:@"email"];
    NSString* phone = [AFLGet type:NSString.class dictionary:arguments key:@"phone"];
    NSDictionary<NSString*, NSString*>* alias = [AFLGet type:NSDictionary.class dictionary:arguments key:@"alias"];
    NSDictionary<NSString*, NSObject*>* attributes = [AFLGet type:NSDictionary.class dictionary:arguments key:@"attributes"];
    
    [AirBridge.state setUserID:ID];
    [AirBridge.state setUserEmail:email];
    [AirBridge.state setUserPhone:phone];
    [AirBridge.state setUserAlias:alias];
    [AirBridge.state setUserAttributes:attributes];
    
    result(nil);
}

- (void) updateUser:(nullable NSDictionary*)arguments result:(FlutterResult)result {
    NSString* ID = [AFLGet type:NSString.class dictionary:arguments key:@"id"];
    NSString* email = [AFLGet type:NSString.class dictionary:arguments key:@"email"];
    NSString* phone = [AFLGet type:NSString.class dictionary:arguments key:@"phone"];
    NSDictionary<NSString*, NSString*>* alias = [AFLGet type:NSDictionary.class dictionary:arguments key:@"alias"];
    NSDictionary<NSString*, NSObject*>* attributes = [AFLGet type:NSDictionary.class dictionary:arguments key:@"attributes"];
    
    if (!ID) {
        [AirBridge.state setUserID:ID];
    }
    if (!email) {
        [AirBridge.state setUserEmail:email];
    }
    if (!phone) {
        [AirBridge.state setUserPhone:phone];
    }
    if (!alias) {
        [AirBridge.state setUserAlias:alias];
    }
    if (!attributes) {
        [AirBridge.state setUserAttributes:attributes];
    }
    
    result(nil);
}

- (void) getDeviceUUID:(FlutterResult)result {
    result(AirBridge.deviceUUID);
}

- (void) startTracking:(FlutterResult)result {
    [AirBridge startTracking];
    result(nil);
}

@end

NS_ASSUME_NONNULL_END
