import '../event/event.dart';
import '../event/add_to_cart_event.dart';
import '../event/purchase_event.dart';
import '../event/sign_in_event.dart';
import '../event/sign_out_event.dart';
import '../event/sign_up_event.dart';
import '../event/view_home_event.dart';
import '../event/view_product_detail_event.dart';
import '../event/view_product_list_event.dart';
import '../event/view_search_result_event.dart';
import '../util/log.dart';
import '../util/map_util.dart';
import 'package:flutter/services.dart';

class EventAPI {
    final MethodChannel _method;

    EventAPI(this._method);

    void send(Event event) {
        if (event == null) {
            Log.isNull('event');
            Log.fail('send');
            return;
        }

        Log.info('send ${event.category}');

        final data = event.toMap();
        MapUtil.cleanNull(data);

        switch(event.category) {
            case ViewHomeEvent.CATEGORY:
                _method.invokeMethod('sendViewHome', data);
                break;

            case ViewSearchResultEvent.CATEGORY:
                _method.invokeMethod('sendViewSearchResult', data);
                break;

            case ViewProductListEvent.CATEGORY:
                _method.invokeMethod('sendViewProductList', data);
                break;

            case ViewProductDetailEvent.CATEGORY:
                _method.invokeMethod('sendViewProductDetail', data);
                break;

            case AddToCartEvent.CATEGORY:
                _method.invokeMethod('sendAddToCart', data);
                break;

            case PurchaseEvent.CATEGORY:
                _method.invokeMethod('sendPurchase', data);
                break;

            case SignInEvent.CATEGORY:
                _method.invokeMethod('sendSignIn', data);
                break;

            case SignUpEvent.CATEGORY:
                _method.invokeMethod('sendSignUp', data);
                break;

            case SignOutEvent.CATEGORY:
                _method.invokeMethod('sendSignOut', data);
                break;

            default:
                _method.invokeMethod('sendCustom', data);
                break;
        }
    }
}
